@@ -2,19 +2,21 @@
 
 =head1 NAME
 
-App::WRT - WRiting Tool, a static site generator and related utilities
+App::WRT - WRiting Tool, a static site/blog generator and related utilities
 
 =for HTML <a href="https://travis-ci.org/brennen/wrt"><img src="https://travis-ci.org/brennen/wrt.svg?branch=master"></a>
 
 =head1 SYNOPSIS
 
-    $ wrt display 2016 > 2016.html
+Using the commandline tools:
 
-Or:
+    $ mkdir project
+    $ cd project
+    $ wrt init         # set up some defaults
+    $ wrt display new  # print html for new posts to stdout
+    $ wrt render-all   # publish html to project/public/
 
-    $ wrt render
-
-Or:
+Using App::WRT in library form:
 
     #!/usr/bin/env perl
 
@@ -28,24 +30,37 @@ Or:
 
 =head1 INSTALLING
 
-It's possible this may run on a Perl as old as 5.10.0, although in practice I
-imagine that at least some of its dependencies have more recent requirements.
-In practice, I know that it works under 5.20.2.  It should work on any
+It's possible but not likely this would run on a Perl as old as 5.10.0.  In
+practice, I know that it works under 5.20.2.  It should be fine on any
 reasonably modern Linux distribution, and may also be fine on MacOS or a BSD of
 your choosing.
 
+To install the latest development version from the main repo:
+
+    $ git clone https://github.com/brennen/wrt.git
+    $ cd wrt
     $ perl Build.PL
     $ ./Build installdeps
     $ ./Build test
     $ ./Build install
 
+To install the latest version released on CPAN:
+
+    $ cpanm App::WRT
+
+Or:
+
+    $ cpan -i App::WRT
+
+You will likely need to use C<sudo> or C<su> to get a systemwide install.
+
 =head1 DESCRIPTION
 
 This started life as C<display.pl>, a simple script to concatenate fragments of
-handwritten HTML by date.  It has since haphazardly accumulated several of the
-usual weblog features (lightweight markup, feed generation, embedded Perl,
-poetry tools, image galleries, and ill-advised dependencies), but the basic
-idea hasn't changed that much.
+handwritten HTML by date.  It has since accumulated several of the usual weblog
+features (lightweight markup, feed generation, embedded Perl, poetry tools,
+image galleries, and ill-advised dependencies), but the basic idea hasn't
+changed that much.
 
 The C<wrt> utility now generates static HTML files, instead of expecting to
 run as a CGI script.  This is a better idea, for the most part.
@@ -58,16 +73,31 @@ By default, entries are stored in a simple directory tree under C<entry_dir>.
 Like:
 
      archives/2001/1/1
-     archives/2001/1/1/sub_entry
+     archives/2001/1/2/index
+     archives/2001/1/2/sub_entry
 
-It is possible (although not as flexible as it ought to be) to redefine the
-directory layout. More about this after a bit.
+Which will publish files like so:
+
+     public/index.html
+     public/all/index.html
+     public/2001/index.html
+     public/2001/1/index.html
+     public/2001/1/1/index.html
+     public/2001/1/2/index.html
+     public/2001/1/2/sub_entry/index.html
+
+Contents will be generated for each year and for the entire collection of dated
+entries.  Month indices will consist of all entries for that month.  A
+top-level index file will consist of the most recent month's entries.
+
+It's possible (although not as flexible as it ought to be) to redefine the
+directory layout.  (See C<%default{entry_map}> below.)
 
 An entry may be either a plain text file, or a directory containing several
-files. If it's a directory, a file named "index" will be treated as the text
+files.  If it's a directory, a file named "index" will be treated as the text
 of the entry, and all other lower-case filenames without extensions will be
 treated as sub-entries or documents within that entry, and displayed
-accordingly. Links to certain other filetypes will be displayed as well.
+accordingly.  Links to certain other filetypes will be displayed as well.
 
 Directories may be nested to an arbitrary depth, although it's probably not a
 good idea to go very deep with the current display logic.
@@ -79,7 +109,7 @@ A PNG or JPEG file with a name like
     2001/1/1/whatever.icon.png
     2001/1/1/whatever/index.icon.png
 
-will be treated as an icon for the appropriate entry file.
+will be treated as an icon for the corresponding entry file.
 
 =head2 MARKUP
 
@@ -89,7 +119,10 @@ thereof. Actually, an entry may consist of any darn thing you please, as long
 as Perl will agree that it is text, but presumably you're going to be feeding
 this to a browser.
 
-Special markup is indicated by a variety of HTML-like container tags.
+Header tags (<h1>, <h2>, etc.) will be used to display titles in feeds and
+other places.
+
+Other special markup is indicated by a variety of HTML-like container tags.
 
 B<Embedded Perl> - evaluated and replaced by whatever value you return
 (evaluated in a scalar context):
@@ -106,13 +139,23 @@ object, for the moment:
 
      <p>The title is <em>${title}</em>.</p>
 
-This will change.
+This is likely to change at some point, so don't build anything too elaborate
+on it.
 
-Embedded code and variables are intended for use in the F<template> file, where
-it's handy to drop in titles or conditionalize aspects of a layout. You want to
-be careful with this sort of thing - it's useful in small doses, but it's also
-a maintainability nightmare waiting to happen.  (WordPress, I am looking at
-you.)
+Embedded code and variables are intended only for use in the F<template> file,
+where it's handy to drop in titles or conditionalize aspects of a layout. You
+want to be careful with this sort of thing - it's useful in small doses, but
+it's also a maintainability nightmare waiting to happen.  (WordPress, I am
+looking at you.)
+
+B<Includes> - replaced by the contents of the enclosed file path, from the
+root of the current wrt project:
+
+    <include>path/to/file</include>
+
+This is a bit constraining, since it doesn't allow for files outside of the
+current project, but is useful for including HTML generated by an external
+script in a page.
 
 B<Several forms of lightweight markup>:
 
@@ -140,11 +183,37 @@ B<And a couple of shortcuts>:
 
 As it stands, freeverse, image, and list are not particularly robust.
 
+=head2 TEMPLATES
+
+A single template, specified by the C<template_dir> and C<template> config
+values, is used to render all pages.  See F<example/templates/basic> for an
+example, or run C<wrt init> in an empty directory and look at
+F<templates/default>.
+
+Here's a short example:
+
+    <!DOCTYPE html>
+    <html>
+    <head>
+      <meta charset="UTF-8">
+      <title>${title_prefix} - ${title}</title>
+    </head>
+
+    <body>
+    ${content}
+    </body>
+
+    </html>
+
+Within templates, C<${foo}> will be replaced with the corresponding
+configuration value.  C<${content}> will always be set to the content of the
+current entry.
+
 =cut
 
 package App::WRT;
 
-use version; our $VERSION = version->declare("v4.0.1");
+use version; our $VERSION = version->declare("v4.2.0");
 
 use strict;
 use warnings;
@@ -153,6 +222,7 @@ no  warnings 'uninitialized';
 use base 'App::WRT::MethodSpit';
 
 use Cwd;
+use File::Spec;
 use HTML::Entities;
 use JSON;
 use XML::Atom::SimpleFeed;
@@ -164,13 +234,66 @@ use App::WRT::Markup   qw(line_parse image_markup eval_perl);
 use App::WRT::Renderer qw(render);
 use App::WRT::Util     qw(dir_list get_date);
 
-=head1 CONFIGURATION
+=head2 CONFIGURATION
+
+Configuration is read from a F<wrt.json> in the directory where the C<wrt>
+utility is invoked, or can (usually) be specified with the C<--config> option.
+
+See F<example/wrt.json> for a sample configuration.
+
+Under the hood, configuration is done by combining a hash called C<%default>
+with values pulled out of the JSON file.  Most defaults can be overwritten
+from the config file, but changing some would require writing Perl, since
+they contain things like subroutine references.
+
+=cut
 
 =over
 
-=item options
+=item %default
 
-See F<example/wrt.json> for a sample configuration.
+Here's a verbatim copy of C<%default>, with some commentary about values.
+
+    my %default = (
+      root_dir       => '.',         # dir for wrt repository
+      entry_dir      => 'archives',  # dir for entry files
+      publish_dir    => 'public',    # dir to publish site to
+      url_root       => "$0?",       # root URL for building links
+      image_url_root => '',          # same for images
+      template_dir   => 'templates', # dir for template files
+      template       => 'default',   # template to use
+      title          => '',          # current title (used in template)
+      title_prefix   => '',          # a string to slap in front of titles
+      stylesheet_url => undef,       # path to a CSS file (used in template)
+      favicon_url    => undef,       # path to a favicon (used in template)
+      feed_alias     => 'feed',      # what entry path should correspond to feed?
+      author         => undef,       # author name (used in template, feed)
+      description    => undef,       # site description (used in template)
+      content        => undef,       # place to stash content for templates
+      embedded_perl  => 1,           # evaluate embedded <perl> tags?
+      default_entry  => 'new',       # what to display if no entry specified
+
+      # A license string for site content:
+      license        => 'public domain', 
+
+      # A string value to replace all pages with (useful for occasional
+      # situations where every page of a site should serve some other
+      # content in-place, like Net Neutrality protest blackouts):
+      overlay        => undef,
+
+      # List of years for the menu:
+      year_list      => [ reverse(1997..(get_date('year') + 1900)) ],
+
+      # What gets considered an entry _path_:
+      entrypath_expr => qr/^ ([a-z0-9_\/-]+) $/x,
+
+      # What gets considered a subentry file (slightly misleading
+      # terminology here):
+      subentry_expr => qr/^[0-9a-z_-]+(\.(tgz|zip|tar[.]gz|gz|txt))?$/,
+
+      # We'll show links for these, but not display them inline:
+      binfile_expr   => qr/[.](tgz|zip|tar[.]gz|gz|txt|pdf)$/,
+    );
 
 =cut
 
@@ -182,17 +305,24 @@ my %default = (
   image_url_root => '',          # same for images
   template_dir   => 'templates', # dir for template files
   template       => 'default',   # template to use
-  title          => '',
-  title_prefix   => '',
-  stylesheet_url => undef,
-  favicon_url    => undef,
-  feed_alias     => 'feed',
-  author         => undef,
-  description    => undef,
+  title          => '',          # current title (used in template)
+  title_prefix   => '',          # a string to slap in front of titles
+  stylesheet_url => undef,       # path to a CSS file (used in template)
+  favicon_url    => undef,       # path to a favicon (used in template)
+  feed_alias     => 'feed',      # what entry path should correspond to feed?
+  author         => undef,       # author name (used in template, feed)
+  description    => undef,       # site description (used in template)
   content        => undef,       # place to stash content for templates
   embedded_perl  => 1,           # evaluate embedded <perl> tags?
-  default_entry  => 'new',
-  license        => 'public domain',
+  default_entry  => 'new',       # what to display if no entry specified
+
+  # A license string for site content:
+  license        => 'public domain', 
+
+  # A string value to replace all pages with (useful for occasional
+  # situations where every page of a site should serve some other
+  # content in-place, like Net Neutrality protest blackouts):
+  overlay        => undef,
 
   # List of years for the menu:
   year_list      => [ reverse(1997..(get_date('year') + 1900)) ],
@@ -200,17 +330,18 @@ my %default = (
   # What gets considered an entry _path_:
   entrypath_expr => qr/^ ([a-z0-9_\/-]+) $/x,
 
-  # What gets considered a subentry file (slightly misleading terminology here):
+  # What gets considered a subentry file (slightly misleading
+  # terminology here):
   subentry_expr => qr/^[0-9a-z_-]+(\.(tgz|zip|tar[.]gz|gz|txt))?$/,
 
   # We'll show links for these, but not display them inline:
   binfile_expr   => qr/[.](tgz|zip|tar[.]gz|gz|txt|pdf)$/,
 );
 
-=item entry_map(\%map)
+=item $default{entry_map}
 
-Takes a hashref which will dispatch entries matching various regexen to
-the appropriate output methods. The default looks something like this:
+A hash which will dispatch entries matching various regexen to the appropriate
+output methods. The default looks something like this:
 
     nnnn/[nn/nn/]doc_name - a document within a day.
     nnnn/nn/nn            - a specific day.
@@ -251,9 +382,9 @@ $default{entry_map} = {
   qr'^[a-z_]'                => sub { entry_stamped (@_, 'all'  ) },
 };
 
-=item entry_descriptions(\%descriptions)
+=item $default{entry_descriptions}
 
-Takes a hashref which contains a map of entry titles to entry descriptions.
+A hashref which contains a map of entry titles to entry descriptions.
 
 =cut
 
@@ -273,9 +404,11 @@ __PACKAGE__->methodspit( keys %default );
 
 =back
 
-=head1 METHODS
+=head2 METHODS AND INTERNALS
 
-For no bigger than this thing is, it gets a little convoluted.
+For no bigger than this thing is, the internals are convoluted.  (This is
+because it's spaghetti code originally written in a now-archaic language by a
+teenager who didn't know how to program.)
 
 =over
 
@@ -319,7 +452,8 @@ sub new {
   my $class = shift;
   my %params = @_;
 
-  my $self = \%default;
+  my %copy_of_default = %default;
+  my $self = \%copy_of_default;
   bless $self, $class;
 
   $self->configure(%params);
@@ -357,10 +491,14 @@ sub display {
 
   # Wrap entries in template:
   my $rendered_page;
-  if ($self->{overlay}) {
-    $rendered_page .= $self->{overlay};
+  if (length($self->overlay)) {
+    $rendered_page .= $self->overlay;
   } else {
-    $rendered_page .= $self->fragment_slurp($self->template_dir . '/' . $self->template);
+    my $template_path = File::Spec->catfile($self->template_dir, $self->template);
+    unless (-f $template_path) {
+      die("$template_path does not exist or is not a plain file");
+    }
+    $rendered_page .= $self->fragment_slurp($template_path);
   }
 
   return $rendered_page;
@@ -408,7 +546,7 @@ sub handle {
 
 =item expand_option($option)
 
-Expands/converts 'all' and 'new' to appropriate values.
+Expands/converts 'all', 'new', and 'fulltext' to appropriate values.
 
 =cut
 
@@ -429,7 +567,6 @@ sub expand_option {
   }
 }
 
-
 =item recent_month()
 
 Tries to find the most recent month in the archive.
@@ -462,9 +599,9 @@ sub recent_month {
   }
 }
 
-=item fulltext
+=item fulltext()
 
-The full text of all entries, in order.
+Returns the full text of all entries, in order.
 
 =cut
 
@@ -714,7 +851,7 @@ sub month {
   return $result;
 }
 
-=item entry_wrapped
+=item entry_wrapped($entry, $level)
 
 Wraps entry() in entry_markup.
 
@@ -727,7 +864,7 @@ sub entry_wrapped {
   return entry_markup($self->entry($entry, $level));
 }
 
-=item entry_stamped
+=item entry_stamped($entry, $level)
 
 Wraps entry() + a datestamp in entry_markup()
 
@@ -743,7 +880,7 @@ sub entry_stamped {
   );
 }
 
-=item entry_topic_list
+=item entry_topic_list($entry)
 
 Get a list of topics (by tag-* files) for the entry.  This hardcodes a
 p1k3-specific thing, and is dumb.
@@ -829,6 +966,12 @@ sub entry {
   return $result;
 }
 
+=item get_sub_entries($entry_loc)
+
+Returns "sub entries" based on the C<subentry_expr> regexp.
+
+=cut
+
 sub get_sub_entries {
   my $self = shift;
   my ($entry_loc) = @_;
@@ -839,6 +982,13 @@ sub get_sub_entries {
               dir_list($entry_loc, 'alpha', $self->subentry_expr);
 }
 
+=item list_contents($entry, @entries)
+
+Returns links (potentially with icons) for a set of sub-entries within an
+entry.
+
+=cut
+
 sub list_contents {
   my $self = shift;
   my ($entry) = shift;
@@ -858,7 +1008,7 @@ sub list_contents {
   return p( em('more:') . " $contents" ) . "\n";
 }
 
-=item icon_markup
+=item icon_markup($entry, $alt)
 
 Check if an icon exists for a given entry if so, return markup to include it.
 Icons are PNG or JPEG image files following a specific naming convention:
@@ -914,7 +1064,7 @@ sub icon_markup {
 }
 }
 
-=item datestamp
+=item datestamp($entry)
 
 Returns a nice html datestamp / breadcrumbs for a given entry.
 
@@ -950,8 +1100,7 @@ sub datestamp {
   return p({class => 'datelink'}, $stamp);
 }
 
-
-=item fragment_slurp
+=item fragment_slurp($file)
 
 Read a text fragment, call line_parse() and eval_perl() to take care of funky
 markup and interpreting embedded code, and then return it as a string. Takes
@@ -989,8 +1138,7 @@ sub fragment_slurp {
   );
 }
 
-
-=item month_name
+=item month_name($number)
 
 Turn numeric dates into English.
 
@@ -1020,7 +1168,7 @@ sub root_locations {
   );
 }
 
-=item local_path
+=item local_path($file)
 
 Return an absolute path for a given file. Called by root_locations.
 
@@ -1032,7 +1180,7 @@ sub local_path {
   return $_[0]->entry_dir . '/' . $_[1];
 }
 
-=item feed_print
+=item feed_print($month)
 
 Return an Atom feed of entries for a month. Defaults to the most
 recent month in the archive.
@@ -1099,7 +1247,6 @@ sub feed_print {
   return $feed->as_string;
 }
 
-
 =back
 
 =head1 SEE ALSO
